/**
 * $Id: SQLCodeEditor.java,v 1.2 2009/07/28 22:55:08 juanca Exp $
 * editors.* is part of ACodeEditor
 * Copyright (C) 2009 Juan Carlos Rodrguez-del-Pino. All rights reserved.
 * license GNU/GPL, see LICENSE.txt or http://www.gnu.org/licenses/gpl-2.0.html
 * @author Juan Carlos Rodriguez-del-Pino
 **/
package editors;

import javax.swing.text.MutableAttributeSet;

public class SQLCodeEditor extends GenericCodeEditor {
	private static final long serialVersionUID = 7397583418429961177L;
	enum States {
		REGULAR, IN_COMMENT, IN_LINE_COMMENT, IN_STRING, IN_DSTRING
	};

	protected void initializeReserved() {
		String lista[] = { "CREATE", "TABLE", "DATABASE", "DROP", "ALTER",
				"TRUNCATE", "INSERT", "INTO", "VALUES", "SELECT", "FROM",
				"JOIN", "WHERE", "GROUP", "BY", "DISTINCT", "HAVING", "ORDER",
				"INSERT", "INTO", "UPDATE", "SET", "DELETE", "TO", "INNER",
				"OUTER", "LEFT", "GRANT" };
		reserved.clear();
		for (int i = 0; i < lista.length; i++)
			reserved.add(lista[i]);
	}

	public SQLCodeEditor(Highlights fonts) {
		super(fonts);
		initializeReserved();
	}

	protected boolean isIdentifierChar(char c) {
		return (c >= 'a' && c <= 'z') || (c >= 'A' && c <= 'Z')
				|| (c >= '0' && c <= '9') || (c == '$') || (c == '_');
	}

	protected void showPending(String text, int begin, int end) {
		if (begin > end || end >= text.length())
			return;
		MutableAttributeSet tipo = font.getRegular();
		String pending = text.substring(begin, end + 1);
		if (reserved.contains(pending))
			tipo = font.getReserved();
		highlight(begin, end, tipo);
	}

	public void codeHighlight() {
		States state= States.REGULAR;
		String s = getText();
		int l = s.length();
		int beginPending = 0;
		for (int i = 0; i < l; i++) {
			if ((i % 100 == 0) && getNeedUpdate()) {
				return;
			}
			char current = s.charAt(i);
			char next;
			if (i + 1 < l)
				next = s.charAt(i + 1);
			else
				next = '\0';
			if (current == CR) // Normalizamos los finales de linea
				if (next == LF)
					continue;
				else
					current = LF;
			switch (state) {
			case REGULAR: {
				if (current >= 'A' && current <= 'Z') {
					continue;
				} else {
					if (beginPending < i) {
						showPending(s, beginPending, i - 1);
						beginPending = i;
					}
					if (current == '/' && next == '*') {
						state = States.IN_COMMENT;
						continue;
					}
					if (current == '-' && next == '-') {
						state = States.IN_LINE_COMMENT;
						continue;
					}
					if (current == '"') {
						state = States.IN_DSTRING;
						continue;
					}
					if (current == '\'') {
						state = States.IN_STRING;
					}
					showPending(s, beginPending, i);
					beginPending = i + 1;
				}
				break;
			}
			case IN_LINE_COMMENT: {
				if (current == LF) {
					highlight(beginPending, i, font.getComment());
					beginPending = i + 1;
					state = States.REGULAR;
				}
				break;
			}
			case IN_COMMENT: {
				if (current == '*' && next == '/') {
					highlight(beginPending, i + 1, font.getComment());
					i++;
					beginPending = i + 1;
					state = States.REGULAR;
				}
				break;
			}
			case IN_DSTRING: {
				if (current == '"') {
					if (next == '"') {
						i++;
					} else {
						highlight(beginPending, i, font.getString());
						beginPending = i + 1;
						state = States.REGULAR;
					}
				}
				break;
			}
			case IN_STRING: {
				if (current == '\'') {
					highlight(beginPending, i, font.getString());
					beginPending = i + 1;
					state = States.REGULAR;
				}
				break;
			}
			}
		}
		switch (state) {
		case REGULAR:
			showPending(s, beginPending, l - 1);
			break;
		case IN_DSTRING:
		case IN_STRING:
			highlight(beginPending, l - 1, font.getString());
			break;
		case IN_LINE_COMMENT:
			highlight(beginPending, l - 1, font.getComment());
			break;
		}
		endUpdate();
	}

	public String getType() {
		return "sql";
	}

	public String getTypeName() {
		return "SQL";
	}

}
